"""Utilities for sorting variable definitions."""

from enum import Enum

from dapla_metadata.variable_definitions._generated.vardef_client.models.language_string_type import (
    LanguageStringType,
)
from dapla_metadata.variable_definitions._generated.vardef_client.models.owner import (
    Owner,
)
from dapla_metadata.variable_definitions._generated.vardef_client.models.supported_languages import (
    SupportedLanguages,
)
from dapla_metadata.variable_definitions.variable_definition import VariableDefinition


class SortOption(Enum):
    NAME_ASC = "name-ascending"
    NAME_DESC = "name-descending"
    SHORT_NAME_ASC = "short-name-ascending"
    SHORT_NAME_DESC = "short-name-descending"
    OWNER_ASC = "owner-ascending"
    OWNER_DESC = "owner-descending"


# Map enum sort options to field name and set order
MAP_SORT_OPTIONS = {
    SortOption.NAME_ASC: ("name", False),
    SortOption.NAME_DESC: ("name", True),
    SortOption.SHORT_NAME_ASC: ("short_name", False),
    SortOption.SHORT_NAME_DESC: ("short_name", True),
    SortOption.OWNER_ASC: ("owner", False),
    SortOption.OWNER_DESC: ("owner", True),
}


def make_sort_key(field_name: str):
    """Create a robust sort key function for a given object attribute."""

    def key_func(obj: VariableDefinition):
        variable_definition = getattr(obj, field_name, None)
        if variable_definition is None:
            result: str = ""
        if isinstance(variable_definition, LanguageStringType):
            val = getattr(variable_definition, SupportedLanguages.NB)
            result = val.casefold() if isinstance(val, str) else ""

        if isinstance(variable_definition, Owner):
            result = getattr(variable_definition, "team", "")

        if isinstance(variable_definition, str):
            result = variable_definition.casefold()

        return result

    return key_func



# coding: utf-8

# flake8: noqa

"""
Internal Variable Definitions Administration API

 ## Introduction  Variable Definitions are centralized definitions of concrete variables which are typically present in multiple datasets. Variable Definitions support standardization of data and metadata and facilitate sharing and joining of data by clarifying when variables have an identical definition.  ## Maintenance of Variable Definitions This API allows for creation, maintenance and access of Variable Definitions.  ### Ownership Creation and maintenance of variables may only be performed by Statistics Norway employees representing a specific Dapla team, who are defined as the owners of a given Variable Definition. The team an owner represents must be specified when making a request through the `active_group` query parameter. All maintenance is to be performed by the owners, with no intervention from administrators.  ### Status All Variable Definitions have an associated status. The possible values for status are `DRAFT`, `PUBLISHED_INTERNAL` and `PUBLISHED_EXTERNAL`.  #### Draft When a Variable Definition is created it is assigned the status `DRAFT`. Under this status the Variable Definition is:  - Only visible to Statistics Norway employees. - Mutable (it may be changed directly without need for versioning). - Not suitable to refer to from other systems.  This status may be changed to `PUBLISHED_INTERNAL` or `PUBLISHED_EXTERNAL` with a direct update.  #### Published Internal Under this status the Variable Definition is:  - Only visible to Statistics Norway employees. - Immutable (all changes are versioned). - Suitable to refer to in internal systems for statistics production. - Not suitable to refer to for external use (for example in Statistikkbanken).  This status may be changed to `PUBLISHED_EXTERNAL` by creating a Patch version.  #### Published External Under this status the Variable Definition is:  - Visible to the general public. - Immutable (all changes are versioned). - Suitable to refer to from any system.  This status may not be changed as it would break immutability. If a Variable Definition is no longer relevant then its period of validity should be ended by specifying a `valid_until` date in a Patch version.  ### Immutability Variable Definitions are immutable. This means that any changes must be performed in a strict versioning system. Consumers can avoid being exposed to breaking changes by specifying a `date_of_validity` when they request a Variable Definition.  #### Patches Patches are for changes which do not affect the fundamental meaning of the Variable Definition.  #### Validity Periods Validity Periods are versions with a period defined by a `valid_from` date and optionally a `valid_until` date. If the fundamental meaning of a Variable Definition is to be changed, it should be done by creating a new Validity Period.

The version of the OpenAPI document: 0.1
Contact: metadata@ssb.no
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

__version__ = "1.0.0"

# Define package exports
__all__ = [
    "DataMigrationApi",
    "DraftVariableDefinitionsApi",
    "PatchesApi",
    "ValidityPeriodsApi",
    "VariableDefinitionsApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "CompleteView",
    "Contact",
    "CreateDraft",
    "CreatePatch",
    "CreateValidityPeriod",
    "GetVardokVardefMappingById200Response",
    "KlassReference",
    "LanguageStringType",
    "ListVariableDefinitions200ResponseInner",
    "Owner",
    "Problem",
    "RenderedContact",
    "RenderedView",
    "SupportedLanguages",
    "UpdateDraft",
    "VardokIdResponse",
    "VardokVardefIdPairResponse",
    "VariableStatus",
]

# import apis into sdk package
from .api.data_migration_api import DataMigrationApi as DataMigrationApi
from .api.draft_variable_definitions_api import (
    DraftVariableDefinitionsApi as DraftVariableDefinitionsApi,
)
from .api.patches_api import PatchesApi as PatchesApi
from .api.validity_periods_api import ValidityPeriodsApi as ValidityPeriodsApi
from .api.variable_definitions_api import (
    VariableDefinitionsApi as VariableDefinitionsApi,
)

# import ApiClient
from .api_response import ApiResponse as ApiResponse
from .api_client import ApiClient as ApiClient
from .configuration import Configuration as Configuration
from .exceptions import OpenApiException as OpenApiException
from .exceptions import ApiTypeError as ApiTypeError
from .exceptions import ApiValueError as ApiValueError
from .exceptions import ApiKeyError as ApiKeyError
from .exceptions import ApiAttributeError as ApiAttributeError
from .exceptions import ApiException as ApiException

# import models into sdk package
from .models.complete_view import CompleteView as CompleteView
from .models.contact import Contact as Contact
from .models.create_draft import CreateDraft as CreateDraft
from .models.create_patch import CreatePatch as CreatePatch
from .models.create_validity_period import CreateValidityPeriod as CreateValidityPeriod
from .models.get_vardok_vardef_mapping_by_id200_response import (
    GetVardokVardefMappingById200Response as GetVardokVardefMappingById200Response,
)
from .models.klass_reference import KlassReference as KlassReference
from .models.language_string_type import LanguageStringType as LanguageStringType
from .models.list_variable_definitions200_response_inner import (
    ListVariableDefinitions200ResponseInner as ListVariableDefinitions200ResponseInner,
)
from .models.owner import Owner as Owner
from .models.problem import Problem as Problem
from .models.rendered_contact import RenderedContact as RenderedContact
from .models.rendered_view import RenderedView as RenderedView
from .models.supported_languages import SupportedLanguages as SupportedLanguages
from .models.update_draft import UpdateDraft as UpdateDraft
from .models.vardok_id_response import VardokIdResponse as VardokIdResponse
from .models.vardok_vardef_id_pair_response import (
    VardokVardefIdPairResponse as VardokVardefIdPairResponse,
)
from .models.variable_status import VariableStatus as VariableStatus



import json
import pathlib
import shutil

import datadoc_model.all_optional.model as all_optional_model
import pytest

from dapla_metadata.datasets import Datadoc
from tests.datasets.constants import TEST_COPY_VARIABLES_FILEPATH
from tests.datasets.constants import TEST_EXISTING_METADATA_FILE_NAME


def test_copy_variable(
    metadata: Datadoc,
    tmp_path: pathlib.Path,
):
    target_short_name = "pers_id"
    shutil.copy(
        str(TEST_COPY_VARIABLES_FILEPATH), str(tmp_path / "copy_variables.json")
    )
    metadata.copy_variable(str(tmp_path / "copy_variables.json"), target_short_name)

    metadata.write_metadata_document()
    written_document = tmp_path / TEST_EXISTING_METADATA_FILE_NAME

    with pathlib.Path.open(written_document) as f:
        written_metadata = json.loads(f.read())
        datadoc_metadata = written_metadata["datadoc"]["variables"]

        assert datadoc_metadata[0]["short_name"] == target_short_name
        correct_variable = datadoc_metadata[0]
        assert correct_variable["name"] is not None
        assert correct_variable["name"][2]["languageCode"] == "nb"
        assert correct_variable["name"][2]["languageText"] == "Ny persid"


def test_variable_not_in_target_dataset(
    metadata: Datadoc,
    tmp_path: pathlib.Path,
):
    target_short_name = "test_kopiering"
    shutil.copy(
        str(TEST_COPY_VARIABLES_FILEPATH), str(tmp_path / "copy_variables.json")
    )

    with pytest.raises(
        ValueError,
        match=rf"Target variable {target_short_name} does not exist in the metadata document you are copying into!",
    ):
        metadata.copy_variable(str(tmp_path / "copy_variables.json"), target_short_name)


def test_different_source_and_target_short_name(
    metadata: Datadoc,
    tmp_path: pathlib.Path,
):
    source_short_name = "test_kopiering"
    target_short_name = "pers_id"

    shutil.copy(
        str(TEST_COPY_VARIABLES_FILEPATH), str(tmp_path / "copy_variables.json")
    )
    metadata.copy_variable(
        str(tmp_path / "copy_variables.json"), target_short_name, source_short_name
    )

    metadata.write_metadata_document()
    written_document = tmp_path / TEST_EXISTING_METADATA_FILE_NAME

    with pathlib.Path.open(written_document) as f:
        written_metadata = json.loads(f.read())
        datadoc_metadata = written_metadata["datadoc"]["variables"]

        assert datadoc_metadata[0]["short_name"] == source_short_name
        correct_variable = datadoc_metadata[0]
        assert correct_variable["name"] is not None
        assert correct_variable["name"][2]["languageCode"] == "nb"
        assert correct_variable["name"][2]["languageText"] == "Test kopiering"


def test_update_variable_error_handling(metadata: Datadoc):
    target_short_name = "non_existent"
    variable = all_optional_model.Variable(
        short_name="non_existent",
        name=[{"languageCode": "nb", "languageText": "Nytt navn"}],
        data_type=all_optional_model.DataType.STRING,
    )

    with pytest.raises(
        ValueError, match=rf"Variable with short_name '{target_short_name}' not found."
    ):
        metadata._update_variable(target_short_name, variable)  # noqa: SLF001


def test_update_variable_from_outdated_metadata_file(
    metadata: Datadoc,
    tmp_path: pathlib.Path,
):
    target_short_name = "pers_id"
    shutil.copy(
        str(TEST_COPY_VARIABLES_FILEPATH), str(tmp_path / "outdated_metadata_file.json")
    )
    metadata.copy_variable(
        str(tmp_path / "outdated_metadata_file.json"), target_short_name
    )

    metadata.write_metadata_document()
    written_document = tmp_path / TEST_EXISTING_METADATA_FILE_NAME

    with pathlib.Path.open(written_document) as f:
        written_metadata = json.loads(f.read())
        datadoc_metadata = written_metadata["datadoc"]["variables"]

        assert datadoc_metadata[0]["short_name"] == target_short_name
        correct_variable = datadoc_metadata[0]
        assert correct_variable["name"] is not None
        assert correct_variable["name"][2]["languageCode"] == "nb"
        assert correct_variable["name"][2]["languageText"] == "Ny persid"


def test_copy_multiple_variables(
    metadata: Datadoc,
    tmp_path: pathlib.Path,
):
    target_short_name = "pers_id"
    another_target_short_name = "sivilstand"
    shutil.copy(
        str(TEST_COPY_VARIABLES_FILEPATH), str(tmp_path / "copy_variables.json")
    )
    metadata.copy_variable(str(tmp_path / "copy_variables.json"), target_short_name)
    metadata.copy_variable(
        str(tmp_path / "copy_variables.json"),
        another_target_short_name,
    )

    metadata.write_metadata_document()
    written_document = tmp_path / TEST_EXISTING_METADATA_FILE_NAME

    with pathlib.Path.open(written_document) as f:
        written_metadata = json.loads(f.read())
        datadoc_metadata = written_metadata["datadoc"]["variables"]

        checks = [
            (0, target_short_name, "Ny persid"),
            (2, another_target_short_name, "Ny Sivilstand"),
        ]

        for idx, short_name, language_text in checks:
            item = datadoc_metadata[idx]
            assert item["short_name"] == short_name
            assert item["name"] is not None
            assert item["name"][2]["languageCode"] == "nb"
            assert item["name"][2]["languageText"] == language_text



#!/usr/bin/env -S uv run --script --quiet

# /// script
# dependencies = ["nox",]
# ///

import os
import shlex
import shutil
import sys
from pathlib import Path
from textwrap import dedent

import nox

package = "dapla_metadata"
python_versions = ["3.10", "3.11", "3.12", "3.13", "3.14"]
nox.needs_version = ">= 2021.6.6"
nox.options.default_venv_backend = "uv"


def install_with_uv(
    session: nox.Session,
    *,
    groups: list[str] | None = None,
    only_groups: list[str] | None = None,
    all_extras: bool = False,
    locked: bool = True,
) -> None:
    """Install packages using uv, pinned to uv.lock."""
    cmd = ["uv", "sync", "--no-default-groups"]
    if locked:
        cmd.append("--locked")
    if groups:
        for group in groups:
            cmd.extend(["--group", group])
    if only_groups:
        for group in only_groups or []:
            cmd.extend(["--only-group", group])
    if all_extras:
        cmd.append("--all-extras")
    cmd.append(
        f"--python={session.virtualenv.location}"
    )  # Target the nox venv's Python interpreter
    session.run_install(
        *cmd, env={"UV_PROJECT_ENVIRONMENT": session.virtualenv.location}
    )

@nox.session(name="pre-commit", python=python_versions[-1])
def precommit(session: nox.Session) -> None:
    """Lint using pre-commit."""
    install_with_uv(session, only_groups=["dev"])
    args = session.posargs or [
        "run",
        "--all-files",
        "--hook-stage=manual",
        "--show-diff-on-failure",
    ]
    session.run("pre-commit", *args)


@nox.session(python=python_versions[1:])
def mypy(session: nox.Session) -> None:
    """Type-check using mypy."""
    install_with_uv(session, groups=["type_check"])
    args = session.posargs or ["src", "tests"]
    session.run("mypy", *args)
    if not session.posargs:
        session.run("mypy", f"--python-executable={sys.executable}", "noxfile.py")


@nox.session(python=python_versions)
def tests(session: nox.Session) -> None:
    """Run the test suite."""
    install_with_uv(session, groups=["test"])
    try:
        session.run(
            "coverage",
            "run",
            "--parallel",
            "-m",
            "pytest",
            "-o",
            "pythonpath=",
            *session.posargs,
        )
    finally:
        if session.interactive:
            session.notify("coverage", posargs=[])


@nox.session(python=python_versions[-1], default=False)
def coverage(session: nox.Session) -> None:
    """Produce the coverage report."""
    install_with_uv(session, only_groups=["test"])
    args = session.posargs or ["report", "--skip-empty"]
    if not session.posargs and any(Path().glob(".coverage.*")):
        session.run("coverage", "combine")

    session.run("coverage", *args)


@nox.session(python=python_versions[-1])
def typeguard(session: nox.Session) -> None:
    """Runtime type checking using Typeguard."""
    install_with_uv(session, groups=["test"])
    session.run("pytest", f"--typeguard-packages={package}.datasets", *session.posargs)


@nox.session(python=python_versions[-1])
def xdoctest(session: nox.Session) -> None:
    """Run examples with xdoctest."""
    install_with_uv(session, groups=["test"])
    if session.posargs:
        args = [package, *session.posargs]
    else:
        args = [f"--modname={package}", "--command=all"]
        if "FORCE_COLOR" in os.environ:
            args.append("--colored=1")
    session.run("python", "-m", "xdoctest", *args)


@nox.session(name="docs-build", python=python_versions[-1])
def docs_build(session: nox.Session) -> None:
    """Build the documentation."""
    install_with_uv(session, groups=["docs"])
    args = session.posargs or ["docs", "docs/_build"]
    if not session.posargs and "FORCE_COLOR" in os.environ:
        args.insert(0, "--color")
    build_dir = Path("docs", "_build")
    if build_dir.exists():
        shutil.rmtree(build_dir)

    session.run("sphinx-build", *args)


@nox.session(python=python_versions[-1], default=False)
def docs(session: nox.Session) -> None:
    """Build and serve the documentation with live reloading on file changes."""
    install_with_uv(session, only_groups=["docs"])
    args = session.posargs or ["--open-browser", "docs", "docs/_build"]
    build_dir = Path("docs", "_build")
    if build_dir.exists():
        shutil.rmtree(build_dir)

    session.run("sphinx-autobuild", *args)


if __name__ == "__main__":
    nox.main()
